// Include DTSource.h if you want to include all the headers.

#include "DTArguments.h"
#include "DTSaveError.h"

#include "DTDataFile.h"
#include "DTDoubleArray.h"
#include "DTMatlabDataFile.h"
#include "DTDoubleArrayOperators.h"

#include <math.h>

//////////////////////////////////////////////////////////////////////////////
//    DT_RetGroup
//////////////////////////////////////////////////////////////////////////////

struct DT_RetGroup {
    DTDoubleArray values;
    double single;

    void pinfo(void) const;
    void pinfoIndent(string) const;

    static void WriteStructure(DTDataStorage &,string);
};

void DT_RetGroup::pinfo(void) const
{
    pinfoIndent("");
}

void DT_RetGroup::pinfoIndent(string pad) const
{
    cerr << pad << "values = "; values.pinfo();
    cerr << pad << "single = " << single << endl;
}

void DT_RetGroup::WriteStructure(DTDataStorage &output,string name)
{
    output.Save("values",name+"_1N");
    output.Save("NumberList",name+"_1T");

    output.Save("single",name+"_2N");
    output.Save("Real Number",name+"_2T");

    output.Save(2,name+"_N");
    output.Save("Group",name);
}

extern void Write(DTDataStorage &,string name,const DT_RetGroup &);

void Write(DTDataStorage &output,string name,const DT_RetGroup &var)
{
    Write(output,name+"_values",var.values);
    output.Save(var.single,name+"_single");
    Write(output,name,DTDoubleArray()); // So that DataTank can see the variable.
}

//////////////////////////////////////////////////////////////////////////////
//    End of structure definitions.
//////////////////////////////////////////////////////////////////////////////

DT_RetGroup Computation(DTMatlabDataFile file,double format);

int main(int argc,const char *argv[])
{
    DTSetArguments(argc,argv);

    DTDataFile inputFile("Input.dtbin",DTFile::ReadOnly);
    // Read in the input variables.
    DTMatlabDataFile file = DTMatlabDataFile("file.mat",DTFile::ReadOnly);
    double format = inputFile.ReadNumber("format");

    // Write the output.
    DTDataFile outputFile("Output.dtbin",DTFile::NewReadWrite);

    DT_RetGroup computed;

    // The computation.
    clock_t t_before = clock();
    computed = Computation(file,format);
    clock_t t_after = clock();
    double exec_time = double(t_after-t_before)/double(CLOCKS_PER_SEC);

    outputFile.Save("Group","Seq_Var");
    DT_RetGroup::WriteStructure(outputFile,"SeqInfo_Var");
    Write(outputFile,"Var",computed);

    // The execution time.
    outputFile.Save(exec_time,"ExecutionTime");
    outputFile.Save("Real Number","Seq_ExecutionTime");

    // The errors.
    DTSaveError(outputFile,"ExecutionErrors");
    outputFile.Save("StringList","Seq_ExecutionErrors");

    return 0;
}

DT_RetGroup Computation(DTMatlabDataFile file,double format)
{
    DT_RetGroup toReturn;
    
    // Fill the return group based on the file.

    // toReturn.values - List of Numbers
    toReturn.values = file.ReadDoubleArray("list");
    // toReturn.single - Real Number
    toReturn.single = file.ReadNumber("v");

    return toReturn;
}
